function classifyResult = challenge(recordName)
%
% Sample entry for the 2017 PhysioNet/CinC Challenge.
%
% INPUTS:
% recordName: string specifying the record name to process
%
% OUTPUTS:
% classifyResult: integer value where
%                     N = normal rhythm
%                     A = AF
%                     O = other rhythm
%                     ~ = noisy recording (poor signal quality)
%
% To run your entry on the entire training set in a format that is
% compatible with PhysioNet's scoring enviroment, run the script
% generateValidationSet.m
%
% The challenge function requires that you have downloaded the challenge
% data 'training_set' in a subdirectory of the current directory.
%    http://physionet.org/physiobank/database/challenge/2017/
%
% This dataset is used by the generateValidationSet.m script to create
% the annotations on your training set that will be used to verify that
% your entry works properly in the PhysioNet testing environment.
%
%
% Version 1.0
%
%
% Written by: Chengyu Liu and Qiao Li January 20 2017
%             chengyu.liu@emory.edu  qiao.li@emory.edu
%
% Last modified by:
%
%

% classifyResult = 'N'; % default output normal rhythm

%% Load classifier and data
load refBeats
load relevantDict_N
load classifier_bagged_tree_findout_noise_adasyn
load classifier_bagged_tree_mp_A_N_O_adasyn

%% Extract signal features
featureMatrix = extractMPFeatures(recordName, relevantDict_N, refBeats);
oversamp_featureMatrix2 = featureMatrix(1,1:17);
features1 = array2table(oversamp_featureMatrix2);

if oversamp_featureMatrix2(1,2) == 0
    classifyResult = '~';
    return;
end

%% Classify the record
class = classifier_bagged_tree_findout_noise_adasyn.predictFcn(features1);
if class
    oversamp_featureMatrix3 = featureMatrix(1,1:48);
    features2 = array2table(oversamp_featureMatrix3);
    class = classifier_bagged_tree_mp_A_N_O_adasyn.predictFcn(features2);
    class_labels = ['A', 'N', 'O'];
    classifyResult = class_labels(class);
else
    classifyResult = '~';
end

end
