function features = extractMPFeatures(recordName, mpdict, refBeats)
%% Preparing
beatLength = 180;
featureNum = 48;
fs = 300;
features = zeros(1,featureNum);

%% Load signal
load(recordName);
signal = val';

i = 1;
%% Remove head noise
[~, signal, proportion] = CleanByEnergy5(signal, fs);
features(i) = proportion;
i=i+1;

if proportion < 0.3
    return;
end

%% Get representitive beats of the signal
% extract beat matrix
[tmp1, beatMatrix1, ~] = extractBeats(signal, 300, beatLength);
[tmp2, beatMatrix2, ~] = extractBeats(-signal, 300, beatLength);

if size(beatMatrix1,1) < 3 || size(beatMatrix2,1) < 3
    return;
end

% extract beat matrix
% [groups1, mean_beats1] = extractMeanBeats(beatMatrix1, 3, 0.8);
% [groups2, mean_beats2] = extractMeanBeats(beatMatrix2, 3, 0.8);

[groups1, mean_beats1] = extractMeanBeatsDBSCAN(beatMatrix1, 1, 0.9);
[groups2, mean_beats2] = extractMeanBeatsDBSCAN(beatMatrix2, 1, 0.9);

if isempty(mean_beats1) || isempty(mean_beats2)
    return;
end

% compute the corelation between the beats and refrence beats
corr1 = corr(mean_beats1', refBeats');
corr2 = corr(mean_beats2', refBeats');
refCorrMax1 = max(max(corr1));
refCorrMax2 = max(max(corr2));

if refCorrMax1 > refCorrMax2
    rep_beats = mean_beats1;
    beatMatrix = beatMatrix1;
    groups = groups1;
    tmp = tmp1;
    refCorrs = corr1;
else
    rep_beats = mean_beats2;
    beatMatrix = beatMatrix2;
    groups = groups2;
    tmp = tmp2;
    refCorrs = corr2;
end

%% Group features
grouped_beat_index = max(groups);
groups_num = size(groups,1);
grouped_beat_num = length(find(grouped_beat_index > 0));
grouped_beat_prop = grouped_beat_num/length(grouped_beat_index);
features(i) = groups_num;
i=i+1;
features(i) = grouped_beat_prop;
i=i+1;

%% Extract RR features
rr = diff(tmp);
features(i) = max(rr);
i=i+1;
features(i) = min(rr);
i=i+1;
features(i) =mean(rr); %ֵ
i=i+1;
features(i) =median(rr); %ֵ
i=i+1;
features(i) =std(rr)/mean(rr); %׼
i=i+1;
features(i) = steppingOfInterbeatIntervals(rr);
i=i+1;
features(i) = dispersionOfPointsAroundDiagonallineInPoincarePlot(rr);
i=i+1;
[ clusters, mean_inter_interval_dist, median_inter_interval_dist ] = clusterInPoincarePlot( rr );
% features(i) = clusters;
% i=i+1;
features(i) = mean_inter_interval_dist;
i=i+1;
features(i) = median_inter_interval_dist;
i=i+1;

%% Extract beat to beat corrs
corrs_mat = corr(beatMatrix');
for j = 1:size(corrs_mat,1)
    corrs_mat(j,j) = 0;
end
corrs_vector = squareform(corrs_mat);
features(i) = mean(corrs_vector);
i=i+1;
features(i) = median(corrs_vector);
i=i+1;

features(i) = mean(max(refCorrs));
i=i+1;
features(i) = median(max(refCorrs));
i=i+1;
features(i) = max(max(refCorrs));
i=i+1;

%% Extract MP features
coeff1 = zeros(1,30);
coeff2 = zeros(1,30);
if size(rep_beats,1) > 1
    [yfit,r,coeff,iopt,qual] = wmpalg('OMP',rep_beats(1,:),mpdict,...,
        'itermax', 30, 'typeplot','none');
    coeff1(iopt) = coeff;
    residue1 = norm(r,2)/norm(rep_beats(1,:),2);
    [yfit,r,coeff,iopt,qual] = wmpalg('OMP',rep_beats(2,:),mpdict,...,
        'itermax', 30, 'typeplot','none');
    coeff2(iopt) = coeff;
    residue2 = norm(r,2)/norm(rep_beats(2,:),2);    
else
    [yfit,r,coeff,iopt,qual] = wmpalg('OMP',rep_beats(1,:),mpdict,...,
        'itermax', 30, 'typeplot','none');
    coeff1(iopt) = coeff;
    residue1 = norm(r,2)/norm(rep_beats(1,:),2);
    coeff2 = coeff1;
    residue2 = residue1;
end

features(i:i+29) = coeff1;
i = i+30;
features(i) = residue1;
i = i+1;
% features(i:i+29) = coeff2;
% i = i+30;
% features(i) = residue2;

end