function [ groups, mean_beats ] = extractMeanBeats( beatMatrix, minGroupSize, minSimilarity )
groups = [];
mean_beats = [];
group_sizes = [];

beatNum = size(beatMatrix,1);

% compute the Pearson correlation efficients of the beats
% beatCorrs = corr(beatMatrix');
beatCorrs = corr(beatMatrix', 'type', 'Pearson');

ungroupedBeatIndex = ones(1,beatNum);
ungroupedBeat = find(ungroupedBeatIndex > 0,1);
while(~isempty(ungroupedBeat))
    % get the beat indexs of the group to which the current ungroupedBeat
    % belongs
    group = beatCorrs(ungroupedBeat,:) > minSimilarity;
    % set all the corr efficients related to the group to zero.
    beatCorrs(group, :) = 0;
    beatCorrs(:, group) = 0;
    % check if it is a valid group whose size is at least minGroupSize
    groupSize = length(find(group > 0));
    if groupSize >= minGroupSize
        % append the group to the groups matrix
        groups = [groups; group];
        % compute the mean beat of this group
        meanBeat = sum(beatMatrix(group,:))/groupSize;
        mean_beats = [mean_beats; meanBeat];
        group_sizes = [group_sizes; groupSize];
    end
    % update ungroupedBeatIndex
    ungroupedBeatIndex = ungroupedBeatIndex - group;
    % find next ungrouped beat
    ungroupedBeat = find(ungroupedBeatIndex > 0, 1);
end

% sort mean beats by groups size
[group_sizes, I] = sort(group_sizes, 'descend');
mean_beats = mean_beats(I,:);


end

